@@ -46,21 +46,15 @@ use self::mode::ModeImpl;
 use super::bandwidth::Bandwidth;
 use super::bbr::SendTimeState;
 use super::Acked;
+use super::BbrBwLoReductionStrategy;
+use super::BbrParams;
 use super::CongestionControl;
 use super::Lost;
 use super::RttStats;
 
 const MAX_MODE_CHANGES_PER_CONGESTION_EVENT: usize = 4;
 
-#[derive(PartialEq)]
-#[allow(dead_code)]
-enum BwLoMode {
-    Default,
-    MinRttReduction,
-    InflightReduction,
-    CwndReduction,
-}
-
+#[derive(Debug)]
 struct Params {
     // STARTUP parameters.
     /// The gain for CWND in startup.
@@ -168,7 +162,40 @@ struct Params {
     bw_lo_mode: BwLoMode,
 }
 
-const PARAMS: Params = Params {
+impl Params {
+    fn with_overrides(mut self, custom_bbr_settings: &BbrParams) -> Self {
+        macro_rules! apply_override {
+            ($field:ident) => {
+                if let Some(custom_value) = custom_bbr_settings.$field {
+                    self.$field = custom_value;
+                }
+            };
+        }
+
+        apply_override!(startup_cwnd_gain);
+        apply_override!(startup_pacing_gain);
+        apply_override!(full_bw_threshold);
+        apply_override!(startup_full_loss_count);
+        apply_override!(drain_cwnd_gain);
+        apply_override!(drain_pacing_gain);
+        apply_override!(enable_reno_coexistence);
+        apply_override!(probe_bw_probe_up_pacing_gain);
+        apply_override!(probe_bw_probe_down_pacing_gain);
+        apply_override!(probe_bw_cwnd_gain);
+        apply_override!(max_probe_up_queue_rounds);
+        apply_override!(loss_threshold);
+        apply_override!(use_bytes_delivered_for_inflight_hi);
+        apply_override!(decrease_startup_pacing_at_end_of_round);
+
+        if let Some(custom_value) = custom_bbr_settings.bw_lo_reduction_strategy {
+            self.bw_lo_mode = custom_value.into();
+        }
+
+        self
+    }
+}
+
+const DEFAULT_PARAMS: Params = Params {
     startup_cwnd_gain: 2.0,
 
     startup_pacing_gain: 2.773,
@@ -238,6 +265,27 @@ const PARAMS: Params = Params {
     bw_lo_mode: BwLoMode::InflightReduction,
 };
 
+#[derive(Debug, PartialEq)]
+enum BwLoMode {
+    Default,
+    MinRttReduction,
+    InflightReduction,
+    CwndReduction,
+}
+
+impl From<BbrBwLoReductionStrategy> for BwLoMode {
+    fn from(value: BbrBwLoReductionStrategy) -> Self {
+        match value {
+            BbrBwLoReductionStrategy::Default => BwLoMode::Default,
+            BbrBwLoReductionStrategy::MinRttReduction =>
+                BwLoMode::MinRttReduction,
+            BbrBwLoReductionStrategy::InflightReduction =>
+                BwLoMode::InflightReduction,
+            BbrBwLoReductionStrategy::CwndReduction => BwLoMode::CwndReduction,
+        }
+    }
+}
+
 #[derive(Debug)]
 struct Limits<T: Ord> {
     lo: T,
@@ -278,6 +326,7 @@ pub(crate) struct BBRv2 {
     last_sample_is_app_limited: bool,
     has_non_app_limited_sample: bool,
     last_quiescence_start: Option<Instant>,
+    params: Params,
 }
 
 struct BBRv2CongestionEvent {
@@ -340,14 +389,17 @@ impl BBRv2 {
     pub fn new(
         initial_congestion_window: usize, max_congestion_window: usize,
         max_segment_size: usize, smoothed_rtt: Duration,
+        custom_bbr_params: Option<&BbrParams>,
     ) -> Self {
         let cwnd = initial_congestion_window * max_segment_size;
+        let params = if let Some(custom_bbr_settings) = custom_bbr_params {
+            DEFAULT_PARAMS.with_overrides(custom_bbr_settings)
+        } else {
+            DEFAULT_PARAMS
+        };
+
         BBRv2 {
-            mode: Mode::startup(BBRv2NetworkModel::new(
-                PARAMS.startup_cwnd_gain,
-                PARAMS.startup_pacing_gain,
-                PARAMS.overestimate_avoidance,
-            )),
+            mode: Mode::startup(BBRv2NetworkModel::new(&params)),
             cwnd,
             pacing_rate: Bandwidth::from_bytes_and_time_delta(cwnd, smoothed_rtt) *
                 2.885,
@@ -360,12 +412,17 @@ impl BBRv2 {
             has_non_app_limited_sample: false,
             last_quiescence_start: None,
             mss: max_segment_size,
+            params,
         }
     }
 
     fn on_exit_quiescence(&mut self, now: Instant) {
         if let Some(last_quiescence_start) = self.last_quiescence_start.take() {
-            self.mode.do_on_exit_quiescence(now, last_quiescence_start)
+            self.mode.do_on_exit_quiescence(
+                now,
+                last_quiescence_start,
+                &self.params,
+            )
         }
     }
 
@@ -396,14 +453,14 @@ impl BBRv2 {
             return;
         }
 
-        if PARAMS.decrease_startup_pacing_at_end_of_round &&
-            self.mode.pacing_gain() < PARAMS.startup_pacing_gain
+        if self.params.decrease_startup_pacing_at_end_of_round &&
+            self.mode.pacing_gain() < self.params.startup_pacing_gain
         {
             self.pacing_rate = target_rate;
             return;
         }
 
-        if PARAMS.bw_lo_mode != BwLoMode::Default &&
+        if self.params.bw_lo_mode != BwLoMode::Default &&
             self.mode.loss_events_in_round() > 0
         {
             self.pacing_rate = target_rate;
@@ -426,7 +483,10 @@ impl BBRv2 {
             self.cwnd = prior_cwnd + bytes_acked;
         }
 
-        self.cwnd = self.mode.get_cwnd_limits().apply_limits(self.cwnd);
+        self.cwnd = self
+            .mode
+            .get_cwnd_limits(&self.params)
+            .apply_limits(self.cwnd);
         self.cwnd = self.cwnd_limits.apply_limits(self.cwnd);
     }
 
@@ -458,7 +518,7 @@ impl CongestionControl for BBRv2 {
         packet_number: u64, bytes: usize, is_retransmissible: bool,
         rtt_stats: &RttStats,
     ) {
-        if bytes_in_flight == 0 && PARAMS.avoid_unnecessary_probe_rtt {
+        if bytes_in_flight == 0 && self.params.avoid_unnecessary_probe_rtt {
             self.on_exit_quiescence(sent_time);
         }
 
@@ -488,6 +548,7 @@ impl CongestionControl for BBRv2 {
             acked_packets,
             lost_packets,
             &mut congestion_event,
+            &self.params,
         );
 
         // Number of mode changes allowed for this congestion event.
@@ -500,6 +561,7 @@ impl CongestionControl for BBRv2 {
                 lost_packets,
                 &mut congestion_event,
                 self.target_bytes_inflight(),
+                &self.params,
             )
         {
             mode_changes_allowed -= 1;
@@ -517,7 +579,7 @@ impl CongestionControl for BBRv2 {
             self.has_non_app_limited_sample = true;
         }
         if congestion_event.bytes_in_flight == 0 &&
-            PARAMS.avoid_unnecessary_probe_rtt
+            self.params.avoid_unnecessary_probe_rtt
         {
             self.on_enter_quiescence(event_time);
         }
@@ -551,10 +613,14 @@ impl CongestionControl for BBRv2 {
     }
 
     fn update_mss(&mut self, new_mss: usize) {
-        self.cwnd_limits.hi = self.cwnd_limits.hi * new_mss / self.mss;
-        self.cwnd_limits.lo = self.cwnd_limits.lo * new_mss / self.mss;
-        self.cwnd = self.cwnd * new_mss / self.mss;
-        self.initial_cwnd = self.initial_cwnd * new_mss / self.mss;
+        self.cwnd_limits.hi = (self.cwnd_limits.hi as u64 * new_mss as u64 /
+            self.mss as u64) as usize;
+        self.cwnd_limits.lo = (self.cwnd_limits.lo as u64 * new_mss as u64 /
+            self.mss as u64) as usize;
+        self.cwnd =
+            (self.cwnd as u64 * new_mss as u64 / self.mss as u64) as usize;
+        self.initial_cwnd = (self.initial_cwnd as u64 * new_mss as u64 /
+            self.mss as u64) as usize;
         self.mss = new_mss;
     }
 
